#include <stdio.h>
#include <alsa/asoundlib.h>

#if 0
#define ACCESS_TYPE	SND_PCM_ACCESS_RW_INTERLEAVED
#define do_write	snd_pcm_writei
#else
#define ACCESS_TYPE	SND_PCM_ACCESS_MMAP_INTERLEAVED
#define do_write	snd_pcm_mmap_writei
#endif

int main(int argc, char **argv)
{
	snd_pcm_t *pcm;
	snd_pcm_hw_params_t *hw;
	snd_pcm_stream_t stream;
	snd_pcm_uframes_t psize, bsize, ret;
	char *buf;
	struct pollfd pfd;

	if (argc < 5) {
		fprintf(stderr, "aiotest device direction period-size buffer-size\n");
		return 1;
	}
	if (*argv[2] == 'p' || *argv[2] == 'P')
		stream = SND_PCM_STREAM_PLAYBACK;
	else
		stream = SND_PCM_STREAM_CAPTURE;
	psize = atoi(argv[3]);
	bsize = atoi(argv[4]);

	buf = calloc(4, bsize);

	if (snd_pcm_open(&pcm, argv[1], stream, 0)) {
		perror("open");
		return 1;
	}
	
	snd_pcm_hw_params_alloca(&hw);
	snd_pcm_hw_params_any(pcm, hw);
	snd_pcm_hw_params_set_access(pcm, hw, ACCESS_TYPE);
	snd_pcm_hw_params_set_format(pcm, hw, SND_PCM_FORMAT_S16_LE);
	snd_pcm_hw_params_set_channels(pcm, hw, 2);
	snd_pcm_hw_params_set_rate(pcm, hw, 48000, 0);
	snd_pcm_hw_params_set_period_size_near(pcm, hw, &psize, 0);
	snd_pcm_hw_params_set_buffer_size_near(pcm, hw, &bsize);

	if (snd_pcm_hw_params(pcm, hw)) {
		perror("hw_params");
		return 1;
	}
	
	snd_pcm_poll_descriptors(pcm, &pfd, 1);

 again:
	if ((ret = do_write(pcm, buf, psize)) < (snd_pcm_sframes_t)psize ||
	    (ret = do_write(pcm, buf, psize)) < (snd_pcm_sframes_t)psize) {
		perror("initial write");
		fprintf(stderr, "psize = %ld / %ld\n", psize, ret);
		return 1;
	}

	if (snd_pcm_state(pcm) != SND_PCM_STATE_RUNNING) {
		if (snd_pcm_start(pcm)) {
			perror("start");
			return 1;
		}
	}

	for (;;) {
		int err = poll(&pfd, 1, -1);
		if (err < 0) {
			if (snd_pcm_state(pcm) == SND_PCM_STATE_XRUN) {
				fprintf(stderr, "XRUN, recovering\n");
				snd_pcm_recover(pcm, err, 1);
				goto again;
			}
			perror("poll");
			return 1;
		}
		//avail = snd_pcm_avail(pcm);
		err = do_write(pcm, buf, psize);
		if (err < 0) {
			if (snd_pcm_state(pcm) == SND_PCM_STATE_XRUN) {
				fprintf(stderr, "XRUN, recovering\n");
				snd_pcm_recover(pcm, err, 1);
				goto again;
			}
			perror("poll");
			return 1;
		}
	}
}
